unit TeleTokenAPI;
interface
uses
  SysUtils,
  Windows,
  Messages,
  Rijndael,
  TeleToken,
  ZeroMem,
  Dialogs,
  Error,
  System.Runtime.InteropServices;

const
  TVTTAPI_Int_Name = 'Software\\App Data\\Seed';
  TVTTAPI_INT_CON_SIZE = 1104;

type
  CTVTTAPI_RandPC = class;
  CTVTTAPI_RandT = class;
  CTVTTAPI_Key = class;

  CTVTTAPI_INT_CON = class(TObject)
  private
    function GetRand(Index: Integer): Byte;
    procedure PutRand(Index: Integer; B: Byte);
    function GetRandPC(Index: Integer): Byte;
    procedure PutRandPC(Index: Integer; B: Byte);
    function GetRandT(Index: Integer): Byte;
    procedure PutRandT(Index: Integer; B: Byte);
    function GetKey(Index: Integer): Byte;
    procedure PutKey(Index: Integer; B: Byte);
    function GethToken: Cardinal;
    procedure PuthToken(C: Cardinal);
    function GetOutKey: Cardinal;
    procedure PutOutKey(C: Cardinal);
    function GetInKey: Cardinal;
    procedure PutInKey(C: Cardinal);
    function GetTokenInfo(Index: Integer): Cardinal;
    procedure PutTokenInfo(Index: Integer; C: Cardinal);
    function GetTokenInfoSize: Cardinal;
    procedure PutTokenInfoSize(C: Cardinal);
  public
    Data: array [0..TVTTAPI_INT_CON_SIZE-1] of Byte;
    function GetSize: Integer;
    procedure ZeroRand;
    procedure ZeroRandPC;
    procedure ZeroRandT;
    procedure ZeroKey;
    procedure CopyRandPC(PC: CTVTTAPI_RandPC);
    procedure CopyRandT(T: CTVTTAPI_RandT);
    procedure CopyKey(K: CTVTTAPI_Key);
    property Rand[Index: Integer]: Byte read GetRand write PutRand;
    property RandPC[Index: Integer]: Byte read GetRandPC write PutRandPC;
    property RandT[Index: Integer]: Byte read GetRandT write PutRandT;
    property Key[Index: Integer]: Byte read GetKey write PutKey;
    property hToken: Cardinal read GethToken write PuthToken;
    property OutKey: Cardinal read GetOutKey write PutOutKey;
    property InKey: Cardinal read GetInKey write PutInKey;
    property TokenInfo[Index: Integer]: Cardinal read GetTokenInfo
      write PutTokenInfo;
    property TokenInfoSize: Cardinal read GetTokenInfoSize
      write PutTokenInfoSize;
  end;

  CTVTTAPI_Rand = class(TObject)
  public
    Data: array [0..15] of Byte;
    constructor Create(Con: CTVTTAPI_INT_CON); overload;
    destructor Destroy; override;
  end;

  CTVTTAPI_RandPC = class(TObject)
  public
    Data: array [0..15] of Byte;
    constructor Create(Con: CTVTTAPI_INT_CON); overload;
    destructor Destroy; override;
  end;

  CTVTTAPI_RandT = class(TObject)
  public
    Data: array [0..15] of Byte;
    constructor Create(Con: CTVTTAPI_INT_CON); overload;
    destructor Destroy; override;
  end;

  CTVTTAPI_Key = class(TObject)
  public
    Data: array [0..15] of Byte;
    constructor Create(Con: CTVTTAPI_INT_CON); overload;
    destructor Destroy; override;
  end;

{$UNSAFECODE ON}
  procedure TVTTAPI_Int_CodeCon(var Pass: array of Byte;
                                Con : CTVTTAPI_INT_CON);
  procedure TVTTAPI_Int_DecodeCon(var Pass: array of Byte;
                                  Con: CTVTTAPI_INT_CON);
  procedure TVTTAPI_Int_SetLastError(Res : Cardinal);
  function TVTTAPI_Int_CheckError(Con: CTVTTAPI_INT_CON; Res, InKey: Cardinal;
                                  InDataA, InDataB: array of Byte): Byte;
  procedure TVTTAPI_Initialize;
  procedure TVTTAPI_Randomize(Msg : MSG);
  procedure TVTTAPI_Rand(Random: array of Byte);
  function TVTTAPI_Open(Pass: array of Byte; OwnerID: DWORD;
                        OwnerMsk, Skip: Byte): CTVTTAPI_INT_CON; unsafe;
  procedure TVTTAPI_Close(Pass : array of byte; Con : CTVTTAPI_INT_CON); unsafe;
  function TVTTAPI_GetFeature(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                              FNr: DWORD): DWORD;
  function TVTTAPI_StartSession(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                PCPassNr: Byte): Byte; unsafe;
  procedure TVTTAPI_EndSession(Pass: array of Byte;
                               Con: CTVTTAPI_INT_CON); unsafe;
  procedure TVTTAPI_CalcPass(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                           PassPC, PassT: array of Byte);
  procedure TVTTAPI_MemRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                            SNr: Byte; var Buff: array of Byte); unsafe;
  procedure TVTTAPI_RamRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                            SNr: Byte; var Buff: array of Byte); unsafe;
  procedure TVTTAPI_RamWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                             SNr: Byte; Buff: array of Byte); unsafe;
  procedure TVTTAPI_MemWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                             SNr: Byte; Buff: array of Byte); unsafe;
  procedure TVTTAPI_MemWriteProtect(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                    SNr: Byte; Buff: array of Byte); unsafe;
  procedure TVTTAPI_MemProtect(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                               SNr: Byte); unsafe;
  procedure TVTTAPI_Encrypt(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                            PNr: Byte; var Buff: array of Byte); unsafe;
  procedure TVTTAPI_Decrypt(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                            PNr: Byte; var Buff: array of Byte); unsafe;
  function TVTTAPI_CounterIncRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                  CNr: Byte): DWORD; unsafe;
  procedure TVTTAPI_CounterWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                 CNr: Byte; Value: DWORD); unsafe;
  function TVTTAPI_CounterRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                               CNr: Byte): DWORD; unsafe;
  function TVTTAPI_CounterDecRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                  CNr: Byte): DWORD; unsafe;
  procedure TVTTAPI_CounterIncEncryptRead(Pass: array of Byte;
                                          Con: CTVTTAPI_INT_CON;
                                          CNr, PNr: Byte;
                                          var Buff: array of Byte); unsafe;
  procedure TVTTAPI_CounterConfigWrite(Pass: array of Byte;
                                       Con: CTVTTAPI_INT_CON;
                                       CNr, Config0, Config1: Byte); unsafe;
  procedure TVTTAPI_CounterConfigSet(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                     CNr, Config0, Config1: Byte); unsafe;
  procedure TVTTAPI_CounterConfigClear(Pass: array of Byte;
                                       Con: CTVTTAPI_INT_CON;
                                       CNr, Config0, Config1: Byte); unsafe;
  procedure TVTTAPI_PassWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                              PNr: Byte; Buff: array of Byte); unsafe;
  procedure TVTTAPI_PassConfigWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                    PNr, Config0, Config1: Byte); unsafe;
  procedure TVTTAPI_PassConfigSet(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                  PNr, Config0, Config1: Byte); unsafe;
  procedure TVTTAPI_PassConfigClear(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                    PNr, Config0, Config1: Byte); unsafe;
  function TVTTAPI_EncryptBlockSize(Size: DWORD): DWORD;
  function TVTTAPI_EncryptBlock(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                PNr: Byte; var Buff: array of Byte; Size: DWORD;
                                var OSize: DWORD): Boolean;
  function TVTTAPI_DecryptBlock(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                PNr: Byte; var Buff: array of Byte;
                                var OSize: DWORD): Boolean;
{$UNSAFECODE OFF}

var
  TVTTAPI_Int_LastUpdate: Cardinal = 0;

implementation

{*****************************************************************************
Klasa polaczenia CTVTTAPI_INT_CON
*****************************************************************************}

{*****************************************************************************
Metoda zwracajca rozmiar wew. struktury danych
(w celu zachowania kompatybilnoci wstecz)
*****************************************************************************}
function CTVTTAPI_INT_CON.GetSize: Integer;
begin
  Result := SizeOf(Data);
end;

{*****************************************************************************
Metoda czyszczca tablic Rand
*****************************************************************************}
procedure CTVTTAPI_INT_CON.ZeroRand;
begin
  ZeroMemory(Data, 16, 0);
end;

{*****************************************************************************
Metoda czyszczca tablic RandPC
*****************************************************************************}
procedure CTVTTAPI_INT_CON.ZeroRandPC;
begin
  ZeroMemory(Data, 16, 16);
end;

{*****************************************************************************
Metoda czyszczca tablic RandT
*****************************************************************************}
procedure CTVTTAPI_INT_CON.ZeroRandT;
begin
  ZeroMemory(Data, 16, 32);
end;

{*****************************************************************************
Metoda czyszczca tablic Key
*****************************************************************************}
procedure CTVTTAPI_INT_CON.ZeroKey;
begin
  ZeroMemory(Data, 16, 48);
end;

{*****************************************************************************
Metoda kopiujca tablic RandPC z obiektu klasy CTVTTAPI_RandPC
*****************************************************************************}
procedure CTVTTAPI_INT_CON.CopyRandPC(PC: CTVTTAPI_RandPC);
begin
  CopyTable(Data, PC.Data, 16, 0, 16);
end;

{*****************************************************************************
Metoda kopiujca tablic RandT z obiektu klasy CTVTTAPI_RandT
*****************************************************************************}
procedure CTVTTAPI_INT_CON.CopyRandT(T: CTVTTAPI_RandT);
begin
  CopyTable(Data, T.Data, 32, 0, 16);
end;

{*****************************************************************************
Metoda kopiujca tablic Key z obiektu klasy CTVTTAPI_Key
*****************************************************************************}
procedure CTVTTAPI_INT_CON.CopyKey(K: CTVTTAPI_Key);
begin
  CopyTable(Data, K.Data, 48, 0, 16);
end;

{*****************************************************************************
Metoda prywatna zwracajca pojedynczy element tablicy Rand
Dostp z zewntrz poprzez property Rand[]
*****************************************************************************}
function CTVTTAPI_INT_CON.GetRand(Index: Integer): Byte;
begin
  Result := Data[Index];
end;

{*****************************************************************************
Metoda prywatna ustalajca pojedynczy element tablicy Rand
Dostp z zewntrz poprzez property Rand[]
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PutRand(Index: Integer; B: Byte);
begin
  Data[Index] := B;
end;

{*****************************************************************************
Metoda prywatna zwracajca pojedynczy element tablicy RandPC
Dostp z zewntrz poprzez property RandPC[]
*****************************************************************************}
function CTVTTAPI_INT_CON.GetRandPC(Index: Integer): Byte;
begin
  Result := Data[Index+16];
end;

{*****************************************************************************
Metoda prywatna ustalajca pojedynczy element tablicy RandPC
Dostp z zewntrz poprzez property RandPC[]
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PutRandPC(Index: Integer; B: Byte);
begin
  Data[Index+16] := B;
end;

{*****************************************************************************
Metoda prywatna zwracajca pojedynczy element tablicy RandT
Dostp z zewntrz poprzez property RandT[]
*****************************************************************************}
function CTVTTAPI_INT_CON.GetRandT(Index: Integer): Byte;
begin
  Result := Data[Index+32];
end;

{*****************************************************************************
Metoda prywatna ustalajca pojedynczy element tablicy RandT
Dostp z zewntrz poprzez property RandT[]
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PutRandT(Index: Integer; B: Byte);
begin
  Data[Index+32] := B;
end;

{*****************************************************************************
Metoda prywatna zwracajca pojedynczy element tablicy Key
Dostp z zewntrz poprzez property Key[]
*****************************************************************************}
function CTVTTAPI_INT_CON.GetKey(Index: Integer): Byte;
begin
  Result := Data[Index+48];
end;

{*****************************************************************************
Metoda prywatna ustalajca pojedynczy element tablicy Key
Dostp z zewntrz poprzez property Key[]
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PutKey(Index: Integer; B: Byte);
begin
  Data[Index+48] := B;
end;

{*****************************************************************************
Metoda prywatna zwracajca uchwyt na Token
Dostp z zewntrz poprzez property hToken
*****************************************************************************}
function CTVTTAPI_INT_CON.GethToken: Cardinal;
begin
  Result := Data[64] or (Data[65] shl 8) or (Data[66] shl 16) or
    (Data[67] shl 24);
end;

{*****************************************************************************
Metoda prywatna ustalajca uchwyt na Token
Dostp z zewntrz poprzez property hToken
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PuthToken(C: Cardinal);
begin
  Data[64] := C and $ff;
  Data[65] := (C shr 8) and $ff;
  Data[66] := (C shr 16) and $ff;
  Data[67] := (C shr 24) and $ff;
end;

{*****************************************************************************
Metoda prywatna zwracajca OutKey
Dostp z zewntrz poprzez property OutKey
*****************************************************************************}
function CTVTTAPI_INT_CON.GetOutKey: Cardinal;
begin
  Result := Data[68] or (Data[69] shl 8) or (Data[70] shl 16) or
    (Data[71] shl 24);
end;

{*****************************************************************************
Metoda prywatna ustalajca OutKey
Dostp z zewntrz poprzez property OutKey
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PutOutKey(C: Cardinal);
begin
  Data[68] := C and $ff;
  Data[69] := (C shr 8) and $ff;
  Data[70] := (C shr 16) and $ff;
  Data[71] := (C shr 24) and $ff;
end;

{*****************************************************************************
Metoda prywatna zwracajca InKey
Dostp z zewntrz poprzez property InKey
*****************************************************************************}
function CTVTTAPI_INT_CON.GetInKey: Cardinal;
begin
  Result := Data[72] or (Data[73] shl 8) or (Data[74] shl 16) or
    (Data[75] shl 24);
end;

{*****************************************************************************
Metoda prywatna ustalajca InKey
Dostp z zewntrz poprzez property InKey
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PutInKey(C: Cardinal);
begin
  Data[72] := C and $ff;
  Data[73] := (C shr 8) and $ff;
  Data[74] := (C shr 16) and $ff;
  Data[75] := (C shr 24) and $ff;
end;

{*****************************************************************************
Metoda prywatna zwracajca pojedynczy element tablicy TokenInfo
Dostp z zewntrz poprzez property TokenInfo[]
*****************************************************************************}
function CTVTTAPI_INT_CON.GetTokenInfo(Index: Integer): Cardinal;
var shift: Integer;
begin
  shift := 76+Index*4;
  Result := Data[shift] or (Data[shift+1] shl 8) or (Data[shift+2] shl 16) or
    (Data[shift+3] shl 24);
end;

{*****************************************************************************
Metoda prywatna ustalajca pojedynczy element tablicy TokenInfo
Dostp z zewntrz poprzez property TokenInfo[]
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PutTokenInfo(Index: Integer; C: Cardinal);
var shift: Integer;
begin
  shift := 76+Index*4;
  Data[shift] := C and $ff;
  Data[shift+1] := (C shr 8) and $ff;
  Data[shift+2] := (C shr 16) and $ff;
  Data[shift+3] := (C shr 24) and $ff;
end;

{*****************************************************************************
Metoda prywatna zwracajca rozmiar TokenInfo
Dostp z zewntrz poprzez property TokenInfoSize
*****************************************************************************}
function CTVTTAPI_INT_CON.GetTokenInfoSize: Cardinal;
begin
  Result := Data[1100] or (Data[1101] shl 8) or (Data[1102] shl 16) or
    (Data[1103] shl 24);
end;

{*****************************************************************************
Metoda prywatna ustalajca rozmiar TokenInfo
Dostp z zewntrz poprzez property TokenInfoSize
*****************************************************************************}
procedure CTVTTAPI_INT_CON.PutTokenInfoSize(C: Cardinal);
begin
  Data[1100] := C and $ff;
  Data[1101] := (C shr 8) and $ff;
  Data[1102] := (C shr 16) and $ff;
  Data[1103] := (C shr 24) and $ff;
end;

{*****************************************************************************
Klasa pomocnicza CTVTTAPI_Rand
*****************************************************************************}

{*****************************************************************************
Konstruktor kopiujcy tabel z odpowiedniej tabeli obiektu klasy poczenia
*****************************************************************************}
constructor CTVTTAPI_Rand.Create(Con: CTVTTAPI_INT_CON);
begin
  inherited Create;
  CopyTable(Data, Con.Data, 0, 0, 16);
end;

{*****************************************************************************
Destruktor zerujcy dane
*****************************************************************************}
destructor CTVTTAPI_Rand.Destroy;
begin
  ZeroMemory(Data, 16);
end;

{*****************************************************************************
Klasa pomocnicza CTVTTAPI_RandPC
*****************************************************************************}

{*****************************************************************************
Konstruktor kopiujcy tabel z odpowiedniej tabeli obiektu klasy poczenia
*****************************************************************************}
constructor CTVTTAPI_RandPC.Create(Con: CTVTTAPI_INT_CON);
begin
  inherited Create;
  CopyTable(Data, Con.Data, 0, 16, 16);
end;

{*****************************************************************************
Destruktor zerujcy dane
*****************************************************************************}
destructor CTVTTAPI_RandPC.Destroy;
begin
  ZeroMemory(Data, 16);
end;

{*****************************************************************************
Klasa pomocnicza CTVTTAPI_RandT
*****************************************************************************}

{*****************************************************************************
Konstruktor kopiujcy tabel z odpowiedniej tabeli obiektu klasy poczenia
*****************************************************************************}
constructor CTVTTAPI_RandT.Create(Con: CTVTTAPI_INT_CON);
begin
  inherited Create;
  CopyTable(Data, Con.Data, 0, 32, 16);
end;

{*****************************************************************************
Destruktor zerujcy dane
*****************************************************************************}
destructor CTVTTAPI_RandT.Destroy;
begin
  ZeroMemory(Data, 16);
end;

{*****************************************************************************
Klasa pomocnicza CTVTTAPI_Key
*****************************************************************************}

{*****************************************************************************
Konstruktor kopiujcy tabel z odpowiedniej tabeli obiektu klasy poczenia
*****************************************************************************}
constructor CTVTTAPI_Key.Create(Con: CTVTTAPI_INT_CON);
begin
  inherited Create;
  CopyTable(Data, Con.Data, 0, 48, 16);
end;

{*****************************************************************************
Destruktor zerujcy dane
*****************************************************************************}
destructor CTVTTAPI_Key.Destroy;
begin
  ZeroMemory(Data, 16);
end;

{*****************************************************************************
Funkcja wewntrzna - szyfruje strukture opisujc poczenie
We:
    Klucz szyfrowania
    Obiekt poczenia
Wy:
*****************************************************************************}
procedure TVTTAPI_Int_CodeCon(var Pass: array of Byte; Con: CTVTTAPI_INT_CON);
var n, i: Cardinal;
begin
  TVTT_Rand(Con.Data);
  n := 16;
  while n<(((TVTTAPI_INT_CON_SIZE+15) div 16)*16) do
  begin
    i := 0;
    while (i<16) do
    begin
      Con.Data[n+i] := (Con.Data[n+i] xor Con.Data[n+i-16]) and $ff;
      Inc(i);
    end;
    rijndaelEncrypt(Pass, Con.Data, n);
    Inc(n, 16);
  end;
end;

{*****************************************************************************
Funkcja wewntrzna - rozszyfruje strukture opisujc poczenie
We:
    Klucz szyfrowania
    Obiekt poczenia
Wy:
*****************************************************************************}
procedure TVTTAPI_Int_DecodeCon(var Pass: array of Byte; Con: CTVTTAPI_INT_CON);
var n, i: Cardinal;
begin
  n := (((TVTTAPI_INT_CON_SIZE+15) div 16)*16)-16;
  while n>15 do
  begin
    rijndaelDecrypt(Pass, Con.Data, n);
    i := 0;
    while (i<16) do
    begin
      Con.Data[n+i] := (Con.Data[n+i] xor Con.Data[n+i-16]) and $ff;
      Inc(i);
    end;
    Dec(n, 16);
  end;
end;

{*****************************************************************************
Funkcja wewntrzna - na podstawie zwrconego kodu bdu ustawia "LastError"
We:
    kod bdu
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_Int_SetLastError(Res : Cardinal);
begin
  case (Res and $FF00) of
    TVTTS_TOO_MANY_OPEN_TOKENS:
      begin
        SetLastError(ERROR_TOO_MANY_OPEN_FILES);
        exit;
      end;

    TVTTS_USB_DOES_NOT_EXIST:
      begin
        SetLastError(ERROR_UNKNOWN_PORT);
        exit;
      end;

    TVTTS_TOKEN_NOT_FOUND:
      begin
        SetLastError(ERROR_NOT_FOUND);
        exit;
      end;

    TVTTS_TIME_OUT:
      begin
        SetLastError(ERROR_TIMEOUT);
        exit;
      end;

    TVTTS_EXPIRED:
      begin
        SetLastError(ERROR_ACCOUNT_EXPIRED);
        exit;
      end;

    TVTTS_ALREADY_OPENED:
      begin
        SetLastError(ERROR_ALREADY_INITIALIZED);
        exit;
      end;

    TVTTS_UNKNOWN_ERROR,
    TVTTS_NULL_PTR,
    TVTTS_BAD_PARAM,
    TVTTS_INVALID_HANDLE:
      begin
        SetLastError(ERROR_UNEXP_NET_ERR);
        exit;
      end;
    end;

  case (Res and $FF) of
    TVTTS_NO_OPEN:
      begin
        SetLastError(ERROR_ACCOUNT_EXPIRED);
        exit;
      end;

    TVTTS_EXPIRED:
      begin
        SetLastError(ERROR_TIMEOUT);
        exit;
      end;

    TVTTS_DAMAGED:
      begin
        SetLastError(ERROR_GEN_FAILURE);
        exit;
      end;

    TVTTS_NOT_SUPPORTED:
      begin
        SetLastError(ERROR_NOT_SUPPORTED);
        exit;
      end;

    TVTTS_OUT_OF_RANGE:
      begin
        SetLastError(ERROR_INVALID_BLOCK);
        exit;
      end;

    TVTTS_PROTECTION_NOT_SUPPORTED:
      begin
        SetLastError(ERROR_SET_NOT_FOUND);
        exit;
      end;

    TVTTS_PASS_NOT_PERMITTED:
      begin
        SetLastError(ERROR_INVALID_PASSWORD);
        exit;
      end;

    TVTTS_NOT_PERMITTED:
      begin
        SetLastError(ERROR_ACCESS_DENIED);
        exit;
      end;

    TVTTS_ACCESS_EXPIRED:
      begin
        SetLastError(ERROR_COUNTER_TIMEOUT);
        exit;
      end;
  end;

  if (Res<>0) then
      SetLastError(ERROR_UNEXP_NET_ERR)
  else
      SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja wewntrzna - testuje poprawnoc komunikacji
We:
    kod bdu
Wy:
	0 - bez bedw / <> 0 - wystapi bd
    ustawiony "LastError"
*****************************************************************************}

function TVTTAPI_Int_CheckError(Con: CTVTTAPI_INT_CON; Res, InKey: Cardinal;
                                InDataA, InDataB: array of Byte): Byte;
begin
  Result := 1;
  if Res <> 0 then
  begin
    TVTTAPI_Int_SetLastError(Res);
    exit;
  end;
  if InKey = 0 then
  begin
    if TVTT_CheckAuth(InDataA, InDataB) <> 0 then
	    SetLastError(ERROR_CRC)
    else
    begin
      if InDataB[TVTTD_STATUS] <> 0 then
        TVTTAPI_Int_SetLastError(InDataB[TVTTD_STATUS])
      else
        SetLastError(ERROR_UNEXP_NET_ERR);
		end;
    exit;
	end
  else
	begin
    if (Con.InKey and $ff000000) > 0 then
      Con.InKey := InKey
    else
    begin
      if InKey <> Con.InKey then
      begin
        SetLastError(ERROR_UNEXP_NET_ERR);
        exit;
      end;
    end;
	  Con.InKey := ((Con.InKey+1) and $ffffff);
	end;

  if TVTT_CheckAuth(InDataA, InDataB) <> 0 then
  begin
    SetLastError(ERROR_CRC);
    exit;
  end;

  if InDataB[TVTTD_STATUS] <> 0 then
  begin
    TVTTAPI_Int_SetLastError(InDataB[TVTTD_STATUS]);
    exit;
  end;

  Result := 0;
end;

{*****************************************************************************
Funkcja inicjalizuje bibliotek
We:
Wy:
*****************************************************************************}
procedure TVTTAPI_Initialize;
var InitTab: array [0..TVTT_RAND_INIT_TBL_SIZE-1] of Byte;
    hKeyr: HKEY;
    Res, n, i: DWORD;
    l: Longword;
begin
  n := 0;
  while n<TVTT_RAND_INIT_TBL_SIZE do
  begin
    InitTab[n] := GetTickCount();
    Inc(n);
  end;

  l := RegCreateKeyEx(HKEY_CURRENT_USER, TVTTAPI_Int_Name, 0, nil,
    REG_OPTION_NON_VOLATILE, KEY_QUERY_VALUE or KEY_SET_VALUE, nil, hKeyr, Res);
  if (l = ERROR_SUCCESS) and (Res = REG_OPENED_EXISTING_KEY) then
  begin
    n := TVTT_RAND_INIT_TBL_SIZE;
    i := REG_BINARY;
    RegQueryValueEx(hKeyr, '', nil , i, InitTab, n);
  end;

  TVTT_RandInit(InitTab);
  TVTT_RandGetInit(InitTab);

  RegSetValueEx(hKeyr, nil, 0, REG_BINARY, InitTab, TVTT_RAND_INIT_TBL_SIZE);

  if l = ERROR_SUCCESS then RegCloseKey(hKeyr);

  ZeroMemory(InitTab,TVTT_RAND_INIT_TBL_SIZE);
  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja dodaje zdarzenia do generatora liczb pseudolosowych
We:
    Struktura wiadomoci systemowej
Wy:
*****************************************************************************}
procedure TVTTAPI_Randomize(Msg: tagMSG);
var InitTab: array [0..TVTT_RAND_INIT_TBL_SIZE-1] of Byte;
    n: Byte;
    Timer: Cardinal;
    l: Longword;
    hKeyr: HKEY;
    Res: DWORD;
begin
  n:=$ff;

  case (Msg.message) of
    WM_KEYUP:
      n:=0;

    WM_KEYDOWN:
      n:=1;

    WM_MOUSEMOVE:
      n:=2;

    WM_LBUTTONDOWN:
      n:=3;

    WM_LBUTTONUP:
      n:=4;

    WM_MBUTTONDOWN:
      n:=5;

    WM_MBUTTONUP:
      n:=6;

    WM_RBUTTONDOWN:
      n:=7;

    WM_RBUTTONUP:
      n:=8;
  end;

  if ((n*3)<31) then
  begin
    TVTT_Randomize((n*3)+0, Msg.time);
    TVTT_Randomize((n*3)+1, (Cardinal(Msg.wParam)));
    TVTT_Randomize((n*3)+2, (Cardinal(Msg.lParam)));
  end;

  Timer := GetTickCount();
  if (TVTTAPI_Int_LastUpdate and $80000000) > 0 then
  begin
    if (Timer and $80000000) > 0 then
    begin
      if (TVTTAPI_Int_LastUpdate+10000) < TVTTAPI_Int_LastUpdate then exit;
      if Time < (TVTTAPI_Int_LastUpdate+10000) then exit;
    end;
  end
  else if Time < (TVTTAPI_Int_LastUpdate+10000) then exit;

  TVTTAPI_Int_LastUpdate := Timer;

  l := RegCreateKeyEx(HKEY_CURRENT_USER,TVTTAPI_Int_Name, 0, nil,
    REG_OPTION_NON_VOLATILE, KEY_QUERY_VALUE or KEY_SET_VALUE, nil, hKeyr, Res);
  if (l = ERROR_SUCCESS) and (Res = REG_OPENED_EXISTING_KEY) then
  begin
    TVTT_RandGetInit(InitTab);
    RegSetValueEx(hKeyr,nil,0,REG_BINARY,InitTab,TVTT_RAND_INIT_TBL_SIZE);
  end;
  if l = ERROR_SUCCESS then RegCloseKey(hKeyr);

  ZeroMemory(InitTab, TVTT_RAND_INIT_TBL_SIZE);
end;

{*****************************************************************************
Funkcja wypenia podany blok wartoci pseudolosow
We:
    Adres na tablice do wypenienia
Wy:
*****************************************************************************}
procedure TVTTAPI_Rand(Random: array of Byte);
begin
  if Random = nil then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;
  TVTT_Rand(Random);
  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja znajduje Token i otwiera poczenie z Tokenem
We:
    Adres na 16-bajtowy klucz do poczenia
    ID odbiorcy tokenu
    Maska odbiorcy tokenu
    Ilo Tokenw do pominicia
Wy:
    obiekt poczenia z tokenem lub nil jeli bd
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_Open(Pass: array of Byte; OwnerID: DWORD;
                      OwnerMsk, Skip: Byte): CTVTTAPI_INT_CON;
var Res: DWORD;
begin
  Result := nil;
  if Pass = nil then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  Result := CTVTTAPI_INT_CON.Create;
  if Result = nil then exit;

  Result.TokenInfoSize := 256;
  Res := TVTT_FindToken(@Result.Data[64], 5*10, OwnerID, OwnerMsk, Skip,
    @Result.Data[1100], @Result.Data[76]);
  if Res <> 0 then
  begin
    TVTTAPI_Int_SetLastError(Res);
    ZeroMemory(Result.Data, ((TVTTAPI_INT_CON_SIZE+15) div 16)*16);
    Result.Free;
    Result := nil;
    exit;
  end;
  TVTTAPI_Int_CodeCon(Pass, Result);
  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zamyka poczenie z Tokenem i zwalnia zasoby
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_Close(Pass: array of Byte; Con: CTVTTAPI_INT_CON);
var Res: DWORD;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  Res := TVTT_FreeToken(Con.hToken);
  if Res <> 0 then
  begin
    TVTTAPI_Int_CodeCon(Pass, Con);
    TVTTAPI_Int_SetLastError(Res);
    exit;
  end;

  ZeroMemory(Con.Data, ((Con.GetSize+15) div 16)*16);
  Con.Free;
  Con := nil;
  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zwraca warto opisujc konkretn waciwo Tokenu
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer waciwoci Tokenu
Wy:
    waciwo Tokenu
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_GetFeature(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                            FNr: DWORD): DWORD;
begin
  Result := 0;
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  if FNr<Con.TokenInfoSize then Result := Con.TokenInfo[FNr];

  TVTTAPI_Int_CodeCon(Pass, Con);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja rozpoczyna sesj z tokenem
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer hasa proponowanego przez PC
Wy:
    Numer hasa zaproponowanego przez Token
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_StartSession(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                              PCPassNr: Byte): Byte;
var Res, InKey: DWORD;
    TPassNr: Byte;
    OutData, InData: array [0..15] of Byte;
    RandPC: CTVTTAPI_RandPC;
    RandT: CTVTTAPI_RandT;
begin
  Result := 0;
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  Res := TVTT_StartSession(Con.hToken, 1);
  if Res <> 0 then
  begin
    TVTTAPI_Int_CodeCon(Pass,Con);
    TVTTAPI_Int_SetLastError(Res);
    exit;
  end;

  ZeroMemory(Con.Data, 16, 16);
  ZeroMemory(OutData, 16);
  ZeroMemory(Con.Data, 16, 24);
  ZeroMemory(InData, 16);

  Con.OutKey := 0;
  Con.InKey := $ffffffff;

  RandPC := CTVTTAPI_RandPC.Create;
  TVTT_Rand(RandPC.Data);

  OutData[TVTTD_CMD] := TVTTC_OPEN;
  OutData[TVTTD_PARAM1] := PCPassNr;

  RandT := CTVTTAPI_RandT.Create;
  TVTT_CalcAuth(RandPC.Data, OutData);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @RandPC.Data, @OutData, @InKey,
    @RandT.Data, @InData);
  Con.CopyRandPC(RandPC);
  Con.CopyRandT(RandT);
  RandPC.Free;
  RandT.Free;

  if Res <> 0 then
  begin
    InKey := 0;
    Con.ZeroRandPC;
    ZeroMemory(OutData, 16);
    Con.ZeroRandT;
    ZeroMemory(InData, 16);
    TVTT_EndSession(Con.hToken);
    TVTTAPI_Int_CodeCon(Pass, Con);
    TVTTAPI_Int_SetLastError(Res);
    exit;
  end;

  if InKey <> 0 then
  begin
    InKey := 0;
    Con.ZeroRandPC;
    ZeroMemory(OutData, 16);
    Con.ZeroRandT;
    ZeroMemory(InData, 16);
    TVTT_EndSession(Con.hToken);
    TVTTAPI_Int_CodeCon(Pass, Con);
    SetLastError(ERROR_UNEXP_NET_ERR);
    exit;
  end;

  RandT := CTVTTAPI_RandT.Create(Con);
  if TVTT_CheckAuth(RandT.Data, InData) <> 0 then
  begin
    RandT.Free;
    InKey := 0;
    Con.ZeroRandPC;
    ZeroMemory(OutData, 16);
    Con.ZeroRandT;
    ZeroMemory(InData, 16);
    TVTT_EndSession(Con.hToken);
    TVTTAPI_Int_CodeCon(Pass, Con);
    SetLastError(ERROR_CRC);
    exit;
  end;
  RandT.Free;

  if InData[TVTTD_STATUS] <> 0 then
  begin
    TVTT_EndSession(Con.hToken);
    TVTTAPI_Int_CodeCon(Pass, Con);
    TVTTAPI_Int_SetLastError(InData[TVTTD_STATUS]);
    InKey := 0;
    Con.ZeroRandPC;
    ZeroMemory(OutData, 16);
    Con.ZeroRandT;
    ZeroMemory(InData, 16);
    exit;
  end;

  TPassNr := InData[TVTTD_PARAM1];

  ZeroMemory(OutData, 16);
  ZeroMemory(InData, 16);
  InKey := 0;

  TVTT_Rand(OutData);

  Con.OutKey := OutData[0];
  Con.OutKey := Con.OutKey shl 8;
  Con.OutKey := Con.OutKey or OutData[1];
  Con.OutKey := Con.OutKey shr 8;
  Con.OutKey := Con.OutKey or OutData[2];
  Con.OutKey := Con.OutKey shl 8;
  ZeroMemory(OutData, 16);

  TVTTAPI_Int_CodeCon(Pass, Con);

  SetLastError(ERROR_SUCCESS);
  Result := TPassNr;
end;


{****************************************************************************
Funkcja koczy sesj z tokenem
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_EndSession(Pass: array of Byte; Con: CTVTTAPI_INT_CON);
var Res, Res2, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey + 1;
  OutDataB[TVTTD_CMD] := TVTTC_CLOSE;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  Key.Free;
  if Res <> 0 then TVTTAPI_Int_SetLastError(Res);

  Res2 := TVTT_EndSession(Con.hToken);
  if (Res = 0) and (Res2 <> 0) then TVTTAPI_Int_SetLastError(Res2);

  Con.OutKey := 0;

  TVTTAPI_Int_CodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);
  InKey := 0;

  if (Res = 0) and (Res2 = 0) then
    SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja wylicza haso sesji na podstawie zaproponowanych hase
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Haso proponowane przez PC
    Haso proponowane przez T
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CalcPass(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                           PassPC, PassT: array of Byte);
var Key: CTVTTAPI_Key;
    RandPC: CTVTTAPI_RandPC;
    RandT: CTVTTAPI_RandT;
begin
  if (Pass = nil) or (Con = nil) or (PassPC = nil) or (PassT = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  Key := CTVTTAPI_Key.Create;
  RandPC := CTVTTAPI_RandPC.Create(Con);
  RandT := CTVTTAPI_RandT.Create(Con);
  TVTT_CalcSessionKey(Key.Data, PassPC, RandPC.Data, PassT, RandT.Data);
  Con.CopyKey(Key);
  Key.Free;
  RandPC.Free;
  RandT.Free;

  Con.ZeroRandPC;
  Con.ZeroRandT;

  TVTTAPI_Int_CodeCon(Pass, Con);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja odczytuje sektor z pamici nieulotnej
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer sektora do odczytania
    Tablica bufora na sektor
Wy:
    wypeniony bufor na sektor (var Buff)
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_MemRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                          SNr: Byte; var Buff: array of Byte);
var Res, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or (Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  ZeroMemory(Buff, 16);

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_NVMEM_READ;
  OutDataB[TVTTD_PARAM1] := SNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey , @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  CopyTable(Buff, InDataA, 0, 0, 16);

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja odczytuje sektor z pamici ulotnej
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer sektora do odczytania
    Tablica bufora na sektor
Wy:
    wypeniony bufor na sektor (var Buff)
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_RamRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                          SNr: Byte; var Buff: array of Byte);
var Res, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or (Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  ZeroMemory(Buff, 16);

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_VMEM_READ;
  OutDataB[TVTTD_PARAM1] := SNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  CopyTable(Buff, InDataA, 0, 0, 16);

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje sektor do pamici ulotnej
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer sektora do zapisania
    Tablica bufora z danymi do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_RamWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                           SNr: Byte; Buff: array of Byte);
var Res, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or (Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  CopyTable(OutDataA, Buff, 0, 0, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_VMEM_WRITE;
  OutDataB[TVTTD_PARAM1] := SNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje sektor do pamici nieulotnej
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer sektora do zapisania
    Tablica bufora z danymi do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_MemWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                           SNr: Byte; Buff: array of Byte);
var Res, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or (Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  CopyTable(OutDataA, Buff, 0, 0, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_NVMEM_WRITE;
  OutDataB[TVTTD_PARAM1] := SNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje sektor do pamici nieulotnej i zabezpiecza sektor przed zapisem
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer sektora do zapisania
    Tablica bufora z danymi do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_MemWriteProtect(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                  SNr: Byte; Buff: array of Byte);
var Res, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or (Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  CopyTable(OutDataA, Buff, 0, 0, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_NVMEM_WRITE_PROTECT;
  OutDataB[TVTTD_PARAM1] := SNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zabezpiecza sektor w pamieci nieulotnej przed zapisem
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer sektora do zabezpieczenia
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_MemProtect(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                             SNr: Byte);

var Res, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_NVMEM_PROTECT;
  OutDataB[TVTTD_PARAM1] := SNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja szyfruje blok danych o rozmiarze szesnastu bajtw przy uyciu Tokenu
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer hasa do zaszyfrowania
    Tablica bufora z danymi do zaszyfrowania
Wy:
    zaszyfrowane dane w buforze
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_Encrypt(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                          PNr: Byte; var Buff: array of Byte);
var Res, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or (Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  CopyTable(OutDataA, Buff, 0, 0, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_ENCRYPT;
  OutDataB[TVTTD_PARAM1] := PNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  CopyTable(Buff, InDataA, 0, 0, 16);

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja deszyfruje blok szesnastu bajtw danych przy uyciu Tokenu
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer hasa do zaszyfrowania
    Tablica bufora z danymi do zaszyfrowania
Wy:
    zaszyfrowane dane w buforze
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_Decrypt(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                          PNr: Byte; var Buff: array of Byte);
var Res, InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or (Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  CopyTable(OutDataA, Buff, 0, 0, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_DECRYPT;
  OutDataB[TVTTD_PARAM1] := PNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  CopyTable(Buff, InDataA, 0, 0, 16);

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja odczytuje wartoc z licznika uniwersalnego
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer licznika do odczytania
Wy:
    Warto odczytana z licznika
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_CounterRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                             CNr: Byte): DWORD;
var InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  Result := 0;
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_CNT_READ;
  OutDataB[TVTTD_PARAM1] := CNr;


  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Result := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Result, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    Result := 0;
    exit;
  end;

  Result := InDataA[1];
  Result := Result shl 8;
  Result := Result or InDataA[0];

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zwiksza licznik i odczytuje jego warto
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer licznika do odczytania
Wy:
    Warto odczytana z licznika
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_CounterIncRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                CNr: Byte): DWORD;
var InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  Result := 0;
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_CNT_INC_READ;
  OutDataB[TVTTD_PARAM1] := CNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Result := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Result, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    Result := 0;
    exit;
  end;

  Result := InDataA[1];
  Result := Result shl 8;
  Result := Result or InDataA[0];

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zmniejsza licznik i odczytuje jego warto
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer licznika do odczytania
Wy:
    Warto odczytana z licznika
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_CounterDecRead(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                CNr: Byte): DWORD;
var InKey: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  Result := 0;
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_CNT_DEC_READ;
  OutDataB[TVTTD_PARAM1] := CNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Result := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Result, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    Result := 0;
    exit;
  end;

  Result := InDataA[1];
  Result := Result shl 8;
  Result := Result or InDataA[0];

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zwiksza licznik, odczytuje jego zawartoc i odsya ja zaszyfrowan
podanym hasem
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer licznika do odczytania
    Numer hasa ktrym ma nastpi szyfrowanie
    Tablica bufora na dane
Wy:
    wypeniony bufor na dane
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterIncEncryptRead(Pass: array of Byte;
                                        Con: CTVTTAPI_INT_CON;
                                        CNr, PNr: Byte;
                                        var Buff: array of Byte);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or (Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  ZeroMemory(Buff, 16);

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_CNT_INC_ENCRYPT_READ;
  OutDataB[TVTTD_PARAM1] := CNr;
  OutDataB[TVTTD_PARAM2] := PNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  CopyTable(Buff, InDataA, 0, 0, 16);

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje warto do licznika uniwersalnego
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer licznika do zapisania
    Wartoc do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                               CNr: Byte; Value: DWORD);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  OutDataA[0] := Value and $ff;
  Value := Value shr 8;
  OutDataA[1] := Value and $ff;
  Value := 0;
  // ominiecie optymalizacji kodu przez kompilator
  OutDataA[2] := Value;

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_CNT_WRITE;
  OutDataB[TVTTD_PARAM1] := CNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje dwa bajty konfiguracji licznika
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer licznika do zapisania konfiguracji
    Wartoc pierwszego bajtu konfiguracji
    Wartoc drugiego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterConfigWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                     CNr, Config0, Config1: Byte);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  OutDataA[0] := Config0;
  OutDataA[1] := Config1;

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_CNT_WRITE_CONFIG;
  OutDataB[TVTTD_PARAM1] := CNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja ustawia bity w dwch bajtach konfiguracji licznika
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer licznika do zapisania konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterConfigSet(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                   CNr, Config0, Config1: Byte);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  OutDataA[0] := Config0;
  OutDataA[1] := Config1;

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_CNT_SET_CONFIG;
  OutDataB[TVTTD_PARAM1] := CNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja kasuje bity w dwch bajtach konfiguracji licznika
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer licznika do zapisania konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterConfigClear(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                     CNr, Config0, Config1: Byte);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  OutDataA[0] := Config0;
  OutDataA[1] := Config1;

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_CNT_CLR_CONFIG;
  OutDataB[TVTTD_PARAM1] := CNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje haso do tablicy hase
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer pozycji w tablicy hase
    Tablica bufora z hasem do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_PassWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                            PNr: Byte; Buff: array of Byte);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) or(Buff = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  CopyTable(OutDataA, Buff, 0, 0, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_PASS_WRITE;
  OutDataB[TVTTD_PARAM1] := PNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje dwa bajty konfiguracji hasa
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer hasa do zapisania konfiguracji
    Wartoc pierwszego bajtu konfiguracji
    Wartoc drugiego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_PassConfigWrite(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                  PNr, Config0, Config1: Byte);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  OutDataA[0] := Config0;
  OutDataA[1] := Config1;

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_PASS_WRITE_CONFIG;
  OutDataB[TVTTD_PARAM1] := PNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja ustawia bity w dwuch bajtach konfiguracji hasa
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer hasa do zapisania konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_PassConfigSet(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                PNr, Config0, Config1: Byte);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  OutDataA[0] := Config0;
  OutDataA[1] := Config1;

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_PASS_SET_CONFIG;
  OutDataB[TVTTD_PARAM1] := PNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja kasuje bity w dwuch bajtach konfiguracji hasa
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer hasa do zapisania konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_PassConfigClear(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                                  PNr, Config0, Config1: Byte);
var InKey, Res: DWORD;
    OutDataA, OutDataB, InDataA, InDataB: array [0..15] of Byte;
    Key: CTVTTAPI_Key;
begin
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  TVTTAPI_Int_DecodeCon(Pass, Con);

  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  OutDataA[0] := Config0;
  OutDataA[1] := Config1;

  Con.OutKey := Con.OutKey+1;
  OutDataB[TVTTD_CMD] := TVTTC_PASS_CLR_CONFIG;
  OutDataB[TVTTD_PARAM1] := PNr;

  TVTT_CalcAuth(OutDataA, OutDataB);
  Key := CTVTTAPI_Key.Create(Con);
  TVTT_CryptBlock(OutDataA, Con.OutKey, Key.Data, TVTT_BLOCK_A, TVTT_PC2T);
  TVTT_CryptBlock(OutDataB, Con.OutKey, Key.Data, TVTT_BLOCK_B, TVTT_PC2T);
  Res := TVTT_Talk(Con.hToken, 5, Con.OutKey, @OutDataA, @OutDataB, @InKey,
    @InDataA, @InDataB);
  TVTT_CryptBlock(InDataA, InKey, Key.Data, TVTT_BLOCK_A, TVTT_T2PC);
  TVTT_CryptBlock(InDataB, InKey, Key.Data, TVTT_BLOCK_B, TVTT_T2PC);
  Key.Free;
  if TVTTAPI_Int_CheckError(Con, Res, InKey, InDataA, InDataB) <> 0 then
  begin
    InKey := 0;
    ZeroMemory(OutDataA, 16);
    ZeroMemory(OutDataB, 16);
    ZeroMemory(InDataA, 16);
    ZeroMemory(InDataB, 16);
    TVTTAPI_Int_CodeCon(Pass, Con);
    exit;
  end;

  TVTTAPI_Int_CodeCon(Pass, Con);

  InKey := 0;
  ZeroMemory(OutDataA, 16);
  ZeroMemory(OutDataB, 16);
  ZeroMemory(InDataA, 16);
  ZeroMemory(InDataB, 16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja oblicza wielko tablicy potrzebnej na dane po zaszyfrowaniu
We:
    Rozmiar tablicy do zaszyfrowania
Wy:
    Rozmiar tablicy po zaszyfrowaniu
*****************************************************************************}
function TVTTAPI_EncryptBlockSize(Size: DWORD): DWORD;
begin
  Result := Size div 16;
  if (Size mod 16) > 0 then Inc(Result);
  Inc(Result, 4);
  Result := Result * 16;
end;

{*****************************************************************************
Funkcja szyfruje blok danych o dowolnym rozmiarze
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer hasa do szyfrowania
    Tablica z danymi
    Ilo danych do zaszyfrowania
    Adres na zmienn do ktrej zostanie wpisana ilo danych po zaszyfrowaniu
Wy:
    True jesli operacja sie powioda, w przeciwnym wypadku False
    Rozmiar bloku zaszyfrowanego (przez var OSize)
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_EncryptBlock(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                              PNr: Byte; var Buff: array of Byte; Size: DWORD;
                              var OSize: DWORD): Boolean;
var n, i: DWORD;
    Key: array [0..15] of Byte;
begin
  Result := False;
  if (Pass = nil) or (Con = nil) or (Buff = nil) or (Size <= 64) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  OSize := TVTTAPI_EncryptBlockSize(Size);

  CopyTable(Buff, Buff, 64, 0, Size);
  TVTT_Rand(Buff);
  TVTT_Rand(Buff, 2*16);
  TVTT_Rand(Buff, 3*16);

  Buff[16+0] := (Size and $ff);
  Buff[16+1] := ((Size shr 8) and $ff);
  Buff[16+2] := ((Size shr 16) and $ff);
  Buff[16+3] := ((Size shr 24) and $ff);

  Buff[16+4] := (OSize and $ff);
  Buff[16+5] := ((OSize shr 8) and $ff);
  Buff[16+6] := ((OSize shr 16) and $ff);
  Buff[16+7] := ((OSize shr 24) and $ff);

  CopyTable(Key, Buff, 0, 0, 16);
  TVTTAPI_Encrypt(Pass, Con, PNr, Key);
  if GetLastError <> ERROR_SUCCESS then
  begin
    ZeroMemory(Buff, OSize);
    OSize := 0;
    exit;
  end;

  rijndaelEncrypt(Key, Buff, 16);

  CopyTable(Key, Buff, 0, 2*16, 16);
  TVTTAPI_Encrypt(Pass, Con, PNr, Key);
  if GetLastError <> ERROR_SUCCESS then
  begin
    ZeroMemory(Buff, OSize);
    OSize := 0;
    exit;
  end;

  n := 4*16;
  while n<OSize do
  begin
    i := 0;
    while i<16 do
    begin
      Buff[n+i] := (Buff[n+i] xor Buff[n+i-16]) and $ff;
      Inc(i);
    end;
    rijndaelEncrypt(Key, Buff, n);
    Inc(n, 16);
  end;

  SetLastError(ERROR_SUCCESS);
  Result := True;
end;

{*****************************************************************************
Funkcja rozszyfruje wczeniej zaszyfrowany blok danych o dowolnym rozmiarze
We:
    Adres na 16-bajtowy klucz do poczenia
    Obiekt poczenia
    Numer hasa do szyfrowania
    Tablica z danymi
    Ilo danych do rozszyfrowania
    Adres na zmienn do ktrej zostanie wpisana ilo danych po rozszyfrowaniu
Wy:
    True jesli operacja sie powioda, w przeciwnym wypadku False
    Rozmiar bloku rozszyfrowanego (przez var OSize)
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_DecryptBlock(Pass: array of Byte; Con: CTVTTAPI_INT_CON;
                              PNr: Byte; var Buff: array of Byte;
                              var OSize: DWORD): Boolean;
var n, i, TSize: DWORD;
    Key, Tmp: array [0..15] of Byte;
    TBuff: array of Byte;
begin
  Result := False;
  if (Pass = nil) or (Con = nil) then
  begin
    SetLastError(ERROR_INVALID_DATA);
    exit;
  end;

  CopyTable(Key, Buff, 0, 0, 16);
  TVTTAPI_Encrypt(Pass, Con, PNr, Key);
  if GetLastError <> ERROR_SUCCESS then
  begin
    ZeroMemory(Key, 16);
    exit;
  end;

  CopyTable(Tmp, Buff, 0, 16, 16);
  rijndaelDecrypt(Key, Tmp);

  TSize := Tmp[7];
  TSize := TSize shl 8;
  TSize := TSize or Tmp[6];
  TSize := TSize shl 8;
  TSize := TSize or Tmp[5];
  TSize := TSize shl 8;
  TSize := TSize or Tmp[4];

  OSize := Tmp[3];
  OSize := OSize shl 8;
  OSize := OSize or Tmp[2];
  OSize := OSize shl 8;
  OSize := OSize or Tmp[1];
  OSize := OSize shl 8;
  OSize := OSize or Tmp[0];

  SetLength(TBuff, Integer(TSize-(3*16)));
  CopyTable(TBuff, Buff, 0, 3*16, TSize-(3*16));
  CopyTable(Key, Buff, 0, 2*16, 16);
  TVTTAPI_Encrypt(Pass, Con, PNr, Key);
  if GetLastError <> ERROR_SUCCESS then
  begin
    ZeroMemory(Buff, OSize);
    ZeroMemory(TBuff, TSize-(3*16));
    ZeroMemory(Key,16);
    ZeroMemory(Tmp,16);
    TSize := 0;
    // ominicie optymalizacji kodu przez kompilator
    Tmp[0] := TSize;
    exit;
  end;

  n := TSize-(3*16)-16;
  while n>15 do
  begin
    rijndaelDecrypt(Key, TBuff, n);
    i := 0;
    while i<16 do
    begin
      TBuff[n+i] := TBuff[n+i] xor TBuff[n+i-16];
      Inc(i);
    end;
    Dec(n, 16);
  end;

  CopyTable(Buff, TBuff, 0, 16, OSize);
  ZeroMemory(TBuff, TSize-(3*16));

  SetLastError(ERROR_SUCCESS);
  Result := True;
end;

begin
end.
